//
//	GConst.cpp
//
//

#include "GConst.h"
#include <math.h>

#include "resource.h"
#ifdef __WXMSW__
	#include "word.h"
#else
	#include "word_e.h"
#endif
#include "incident.h"
#include "wx/wx.h"
#include "wx/string.h"
#include "wx/timer.h"
#include "KidsCindyCanvas.h"
#include "KidsCindyFrame.h"
#include "GCalculation.h"

void Meet_CL(CComplex aa,CComplex bb,CComplex cc, CComplex pp,CComplex qq,CComplex rr,CComplex *xx1 ,CComplex *xx2);
bool FindCnFromPt(GConst* cn,GPoint* pt,GConst** cn1);


GConst::GConst(void)
{
	LInput1=LInput2=LInput3=NULL;
	PInput1=PInput2=PInput3=NULL;
	LOutput=NULL;
	POutput=NULL;
	COutput=NULL;
	next=NULL;
	Const_Type=CONST_FREE;
}

GConst::GConst(short c_t)
{
	LInput1=LInput2=LInput3=NULL;
	PInput1=PInput2=PInput3=NULL;
	LOutput=NULL;
	POutput=NULL;
	COutput=NULL;
	next=NULL;
	Const_Type=c_t;
}

GConst::~GConst(void)
{
}

bool GConst::Evaluate()
{
	switch(Const_Type){
	case CONST_FREE:
		if(Evaluate_FreePoint()) return true;
		break;
	case CONST_JOIN:
		if(Evaluate_Join()) return true;
		break;
	case CONST_ON_LINE:
		if(Evaluate_PtOnLine()) return true;
		break;
	case CONST_MEET:
		if(Evaluate_Meet()) return true;
		break;
	case CONST_PARALLEL:
		if(Evaluate_Parallel()) return true;
		break;
	case CONST_PERP:
		if(Evaluate_Perp()) return true;
		break;
	case CONST_MID:
		if(Evaluate_Mid()) return true;
		break;
	case CONST_PT_ON_PARA:
		if(Evaluate_PtOnPara()) return true;
		break;
	case CONST_PT_ON_PERP:
		if(Evaluate_PtOnPerp()) return true;
		break;
	case CONST_MULTI_CIRCLE:
		if(Evaluate_MCircle()) return true;
		break;
	case CONST_PT_ON_CIRCLE:
		if(Evaluate_PtOnCircle()) return true;
		break;
	case CONST_MEET_CL:
		if(Evaluate_MeetCL()) return true;
		break;
	case CONST_MEET_CC:
		if(Evaluate_MeetCC()) return true;
		break;
	case CONST_BISECTOR:
		if(Evaluate_Bisector()) return true;
		break;
	case CONST_3P_BISECTOR:
		if(Evaluate_3pBisector()) return true;
		break;
	case CONST_GRAPH:
		if(Evaluate_Graph()) return true;
		break;
	case CONST_PT_ON_CURVE:
		if(Evaluate_PtOnCurve()) return true;
		break;
	case CONST_TANGENT:
		if(Evaluate_Tangent()) return true;
		break;
	case CONST_COMPASS:
		if(Evaluate_Compass()) return true;
		break;
	case CONST_ORBIT1:  // orbit for a point on a circle
		if(Evaluate_Orbit1()) return true;
		break;
	case CONST_ORBIT2:  // orbit for a point on a line
		if(Evaluate_Orbit2()) return true;
		break;
	case CONST_ORTHOCENTER:
		if(Evaluate_Orthcenter()) return true;
		break;
	case CONST_PT_ON_3P_BISECTOR:
		if(Evaluate_PtOn3PBisector()) return true;
		break;
	case CONST_PT_ON_BISECTOR:
		if(Evaluate_PtOnBisector()) return true;
		break;
	case CONST_PT_ON_TANGENT:
		if(Evaluate_PtOnTangent()) return true;
		break;
	case CONST_CURVE_TANGENT:
		if(Evaluate_CurveTangent()) return true;
		break;
	case CONST_PT_ON_CURVE_TANGENT:
		if(Evaluate_PtOnCurveTangent()) return true;
		break;
	case CONST_PARA_CURVE:
		if(Evaluate_ParaCurve()) return true;
		break;
	}
	if(Output_Type()==OUTPUT_POINT){
		POutput->GetAij();
		POutput->geom_vx=POutput->xz().real() - POutput->geom_oldx;
		POutput->geom_vy=POutput->yz().real() - POutput->geom_oldy;
		POutput->geom_oldx=POutput->xz().real();
		POutput->geom_oldy=POutput->yz().real();
	}
	return false;
}

int GConst::Output_Type(void)
{
	switch(Const_Type){
	case CONST_FREE:
	case CONST_ON_LINE:
	case CONST_MEET:
	case CONST_MID:
	case CONST_PT_ON_PARA:
	case CONST_PT_ON_PERP:
	case CONST_PT_ON_CIRCLE:
	case CONST_MEET_CL:
	case CONST_MEET_CC:
	case CONST_PT_ON_CURVE:
	case CONST_ORTHOCENTER:
	case CONST_PT_ON_3P_BISECTOR:
	case CONST_PT_ON_BISECTOR:
	case CONST_PT_ON_TANGENT:
	case CONST_PT_ON_CURVE_TANGENT:
		return OUTPUT_POINT;break;
	case CONST_JOIN:
	case CONST_PARALLEL:
	case CONST_PERP:
	case CONST_BISECTOR :
	case CONST_3P_BISECTOR:
	case CONST_TANGENT :
	case CONST_CURVE_TANGENT :
		return OUTPUT_LINE;break;
	case CONST_MULTI_CIRCLE:
	case CONST_COMPASS:
		return OUTPUT_CIRCLE;break;
	case CONST_TEXT:
		return OUTPUT_TEXT;break;
	case CONST_GRAPH:
	case CONST_ORBIT1:
	case CONST_ORBIT2:
	case CONST_PARA_CURVE:
		return OUTPUT_CURVE;break;
	case CONST_INVISIBLE:
		return CONST_INVISIBLE;break;
	case CONST_ZOOMIN:
		return CONST_ZOOMIN;break;
	case CONST_ZOOMOUT:
		return CONST_ZOOMOUT;break;
	case CONST_EFFECT_POINT:
	case CONST_EFFECT_LINE:
	case CONST_EFFECT_CIRCLE:
	case CONST_EFFECT_TEXT:
	case CONST_EFFECT_CURVE:
		return OUTPUT_EFFECT;
	}
	return 0;
}


GConst* MyCanvas::NewConst(int consttype)
{
	int i;
	if(NOConstStock>0){
		for(int j=NOConstStock-1;j>=0;j--){
			switch(PushConstStock[j]->Output_Type()){
			case OUTPUT_POINT:
				DeletePoint(PushConstStock[j]->POutput);
				break;
			case OUTPUT_LINE:
				DeleteLine(PushConstStock[j]->LOutput);
				break;
			case OUTPUT_CIRCLE:
				DeleteCircle(PushConstStock[j]->COutput);
				break;
			case OUTPUT_TEXT:
				DeleteText(PushConstStock[j]->TextOutput);
				break;
			case OUTPUT_CURVE:
				DeleteCurve(PushConstStock[j]->CurveOutput);
				break;
			}
		}
		NOConstStock=0;
	}
	for(i=0;i<NUM_OF_POINTS;i++){
		if(constbuffer[i].used==false){
			constbuffer[i].LInput1=constbuffer[i].LInput2=constbuffer[i].LInput3=NULL;
			constbuffer[i].PInput1=constbuffer[i].PInput2=constbuffer[i].PInput3=NULL;
			constbuffer[i].CInput1=constbuffer[i].CInput2=constbuffer[i].CInput3=NULL;
			constbuffer[i].CurveInput1=constbuffer[i].CurveInput2=NULL;
			constbuffer[i].ConstInput1=constbuffer[i].ConstInput2=constbuffer[i].ConstInput3=NULL;
			constbuffer[i].LOutput=NULL;
			constbuffer[i].POutput=NULL;
			constbuffer[i].COutput=NULL;
			constbuffer[i].TextOutput=NULL;
			constbuffer[i].next=NULL;
			constbuffer[i].Const_Type=consttype;
			constbuffer[i].used=true;
			constbuffer[i].geomperdot=geomperdot;
			constbuffer[i].boxleft=boxleft;
			constbuffer[i].boxright=boxright;
			constbuffer[i].boxtop=boxtop;
			constbuffer[i].boxbottom=boxbottom;
			*lastcn=&(constbuffer[i]);
			lastcn=&(constbuffer[i].next);
			sheetbuffer[currentSheet].MyConst=cns;
			sheetbuffer[currentSheet].MyLastConst=lastcn;
			constbuffer[i].MainConst=cns;
			return &(constbuffer[i]);
		}
	}
	return NULL;
}

bool MyCanvas::ConstFree(double gx,double gy,bool hilight)
{
	cursorpt = NewPoint(CComplex(gx,0.0),CComplex(gy,0.0));
	cursorpt->next=NULL;
	Sethilighted(cursorpt);
	GConst* cnp = NewConst(CONST_FREE);
	cnp->POutput=cursorpt;
	cnp->PInput1=cursorpt;
	return false;
}

bool MyCanvas::ConstOnLine(GLine* ln1)
{
	GConst *cnp, *cnp2;
	FindPtFromCn(cns,cursorpt,&cnp);
	FindLnFromCn(cns,ln1,&cnp2);
	GPoint *ptp1, *ptp2;
	GLine *lnp;
	if(cnp2->Const_Type==CONST_JOIN){ //		LINE=JOIN by 2 pts
		ptp1=cnp2->PInput1;
		ptp2=cnp2->PInput2;
		cnp->Const_Type=CONST_ON_LINE;
		cnp->POutput=cursorpt;
		cnp->PInput1=ptp1;
		cnp->PInput2=ptp2;
		CComplex parac1,parac2;
		if(cursorpt->xx==ptp1->xx){
			parac1=cursorpt->yz() - ptp1->yz();
			parac2=ptp2->yz() - cursorpt->yz();
		}
		else {
			parac1=cursorpt->xz() - ptp1->xz();
			parac2=ptp2->xz() - cursorpt->xz();
		}
		if(	parac1 + parac2== CComplex(0.0,0.0)){
			parac1=CComplex(1.0,0.0);
			parac2=CComplex(1.0,0.0);
		}
		cnp->para1=parac1;
		cnp->para2=parac2;
		AssignIncident(ln1,cursorpt,(parac1/(parac1+parac2)).real());
		return true;
	}
	else if(cnp2->Const_Type==CONST_PARALLEL){ // CONST_PARA
		cnp->Const_Type=CONST_PT_ON_PARA;
		cnp->POutput=cursorpt;
		cnp->PInput1=ptp1=cnp2->PInput1;
		cnp->LInput1=lnp=cnp2->LInput1;
		CComplex parac1;
		if(lnp->yy == CComplex(0.0,0.0)){
			parac1 = (cursorpt->yz() - ptp1->yz()) / lnp->xx ;
		}
		else {
			parac1 = (cursorpt->xz() - ptp1->xz()) / lnp->yy * CComplex(-1.0,0.0);
		}
		cnp->para1=parac1;
		AssignIncident(ln1,cursorpt,parac1.real());
		return true;
	}
	else if(cnp2->Const_Type==CONST_PERP){ 
		cnp->Const_Type=CONST_PT_ON_PERP;
		cnp->POutput=cursorpt;
		cnp->PInput1=ptp1=cnp2->PInput1;
		cnp->LInput1=lnp=cnp2->LInput1;
		CComplex parac1;
		if(lnp->yy == CComplex(0.0,0.0)){
			parac1 = (cursorpt->xz() - ptp1->xz()) / lnp->xx ;
		}
		else {
			parac1 = (cursorpt->yz() - ptp1->yz()) / lnp->yy;
		}
		cnp->para1=parac1;
		AssignIncident(ln1,cursorpt,parac1.real());
		return true;
	}
	else if(cnp2->Const_Type==CONST_3P_BISECTOR){
		cnp->Const_Type=CONST_PT_ON_3P_BISECTOR;
		cnp->POutput=cursorpt;
		cnp->PInput1=cnp2->PInput1;
		cnp->PInput2=cnp2->PInput2;
		cnp->PInput3=cnp2->PInput3;
		double g1x=cnp->PInput1->xz().real();
		double g1y=cnp->PInput1->yz().real();
		double g2x=cnp->PInput2->xz().real();
		double g2y=cnp->PInput2->yz().real();
		double g3x=cnp->PInput3->xz().real();
		double g3y=cnp->PInput3->yz().real();
		double norm12 = sqrt((g1x-g2x)*(g1x-g2x)+(g1y-g2y)*(g1y-g2y));
		double norm32 = sqrt((g3x-g2x)*(g3x-g2x)+(g3y-g2y)*(g3y-g2y));
		if(norm12<0.0001){
			norm12=1.0;
			g1x=g2x+1.0;
			g1y=g2y;
		}
		if(norm32<0.0001){
			norm32=1.0;
			g3x=g2x;
			g3y=g2y+1.0;
		}
		g1x=g2x + (g1x-g2x)/norm12;
		g1y=g2y + (g1y-g2y)/norm12;
		g3x=g2x + (g3x-g2x)/norm32;
		g3y=g2y + (g3y-g2y)/norm32;
		double g4x = (g1x+g3x)*0.5 - g2x;
		double g4y = (g1y+g3y)*0.5 - g2y;
		double norm4 = sqrt(g4x*g4x+g4y*g4y);
		if(norm4!=0){
			g4x/=norm4;
			g4y/=norm4;
		}
		else {
			g4x = g1y-g3y;
			g4y = - g1x+g3x;
			norm4 = sqrt(g4x*g4x+g4y*g4y);
			g4x/=norm4;
			g4y/=norm4;
		}
		double para;
		if(g4y==0.0){
			para = (cursorpt->xz().real() - g2x) / g4x;
		}
		else {
			para = (cursorpt->yz().real() - g2y) / g4y;
		}
		cnp->para1=CComplex(para,0.0);
		AssignIncident(ln1,cursorpt,para);
		return true;
	}
	else if(cnp2->Const_Type==CONST_BISECTOR){
		cnp->Const_Type=CONST_PT_ON_BISECTOR;
		cnp->POutput=cursorpt;
		cnp->LInput1=cnp2->LInput1;
		cnp->LInput2=cnp2->LInput2;
		cnp->para1=cnp2->para1;

		CComplex g1x = cnp->LInput1->yy * cnp->LInput2->zz -  cnp->LInput1->zz * cnp->LInput2->yy;
		CComplex g1y = cnp->LInput1->zz * cnp->LInput2->xx -  cnp->LInput1->xx * cnp->LInput2->zz;
		CComplex g1z = cnp->LInput1->xx * cnp->LInput2->yy -  cnp->LInput1->yy * cnp->LInput2->xx;
		CComplex g2x = cnp2->LOutput->xx;
		CComplex g2y = cnp2->LOutput->yy;
		CComplex g2z = cnp2->LOutput->zz;
		CComplex nn = sqrtC(g2x*g2x+g2y*g2y);
		g2x /= nn;
		g2y /= nn;
		if(g1z==CComplex(0.0,0.0)){
			cnp->para2=CComplex(0.0,0.0);
		}
		else if(g2y!=CComplex(0.0,0.0)){
			cnp->para2 = ( cursorpt->xz() - g1x / g1z) / g2y ;
		}
		else {
			cnp->para2 = (g1y / g1z - cursorpt->yz() ) / g2x ;
		}
		AssignIncident(ln1,cursorpt,cnp->para2.real());
		return true;
	}
	else if(cnp2->Const_Type==CONST_TANGENT){
		cnp->Const_Type=CONST_PT_ON_TANGENT;
		cnp->POutput=cursorpt;
		cnp->CInput1=cnp2->CInput1;
		cnp->PInput1=cnp2->PInput1;
		CComplex t1y = cnp->PInput1->xz() - cnp->CInput1->xx;
		CComplex t1x = cnp->CInput1->yy - cnp->PInput1->yz();
		if(t1x!=CComplex(0.0,0.0)){
			cnp->para1 = (cnp->POutput->xz() - cnp->PInput1->xz()) / t1x;
		}
		else if(t1y!=CComplex(0.0,0.0)){
			cnp->para1 = (cnp->POutput->yz() - cnp->PInput1->yz()) / t1y;
		}
		else {
			cnp->para1 = CComplex(0.0,0.0);
		}
		AssignIncident(ln1,cursorpt,cnp->para1.real());
		return true;
	}
	else if(cnp2->Const_Type==CONST_CURVE_TANGENT){
		cnp->Const_Type=CONST_PT_ON_CURVE_TANGENT;
		cnp->POutput=cursorpt;
		cnp->CurveInput1=cnp2->CurveInput1;
		cnp->PInput1=cnp2->PInput1;
		CComplex ax=cnp->PInput1->xz() - cursorpt->xz();
		CComplex ay=cnp->PInput1->yz() - cursorpt->yz();
		CComplex bx= cnp2->LOutput->yy * CComplex(-1.0,0.0) ;
		CComplex by= cnp2->LOutput->xx ;
		CComplex ss=(ax*bx+ay*by)/(bx*bx+by*by);
		cnp->para1=ss;
		AssignIncident(ln1,cursorpt,cnp->para1.real());
		return true;
	}
	return false;
}

bool MyCanvas::ConstOnCircle(GCircle* ci)
{
	GConst *cnp, *cnp2;
	FindPtFromCn(cns,cursorpt,&cnp);
	FindCiFromCn(cns,ci,&cnp2);
	cnp->Const_Type=CONST_PT_ON_CIRCLE;
	cnp->POutput=cursorpt;
	cnp->CInput1=ci;
	double vx=cursorpt->xz().real() - ci->xx.real();
	double vy=cursorpt->yz().real() - ci->yy.real();
	double theta = atan2(-vy,vx);
	cnp->para1=CComplex(theta,0.0);
	AssignIncident(ci,cursorpt,cnp->para1.real());
	Deletehilighted(ci);
	return true;
}

bool MyCanvas::ConstOnCurve(GParaCurve* cu)
{
	GConst *cnp, *cnq;
	FindPtFromCn(cns,cursorpt,&cnp);
	FindCurveFromCns(cns,cu,&cnq);
	cnp->Const_Type=CONST_PT_ON_CURVE;
	cnp->POutput=cursorpt;
	cnp->CurveInput1=cu;
	double gx=cursorpt->xz().real();
	double gy=cursorpt->yz().real();
	double g1x,g1y,g2x,g2y;
	double x0,y0,t=0.0;
	bool snap=false;
	int i;
	g1x=cu->orbitx[0];
	g1y=cu->orbity[0];
	for(i=1;i<cu->orbit_len && !snap;i++){
		g2x=cu->orbitx[i];
		g2y=cu->orbity[i];
		if(g1x!=g2x || g1y!=g2y){
			t = ((gx-g1x)*(g2x-g1x)+(gy-g1y)*(g2y-g1y)) / ((g2x-g1x)*(g2x-g1x)+(g2y-g1y)*(g2y-g1y));
			x0 = g1x*(1-t)+g2x*t;
			y0 = g1y*(1-t)+g2y*t;
			if(t>=0.0 && t<1.0 && (x0- gx)*(x0 - gx)+(y0 - gy)*(y0 - gy) < snapradius2  && cu->invisible==false){
				snap=true;
			}
		}
		g1x=g2x;g1y=g2y;
	}
	if(snap){
		cnp->para1=CComplex(1.0*(i-2)+t,0.0);
	}
	else {
		cnp->para1=CComplex(0.0,0.0);
	}
	Deletehilighted(cu);
	return true;
}

bool MyCanvas::ConstMeet(GLine* ln1, GLine* ln2)
{
	GConst * gcn;
	FindPtFromCn(cns,cursorpt,&gcn);
	gcn->Const_Type=CONST_MEET;
	gcn->POutput=cursorpt;
	gcn->LInput1=ln1;
	gcn->LInput2=ln2;
	if(!CheckCoincidency(cursorpt)){
		AssignIncident(ln1,cursorpt);
		ResetLineIcPara(ln1);
		AssignIncident(ln2,cursorpt);
		ResetLineIcPara(ln2);
		CheckIncidency(cursorpt);
	}
	return false;
}



bool MyCanvas::ConstMeet(GCircle* ci1, GLine* ln1,double gx,double gy)
{
	GConst * cnp;
	FindPtFromCn(cns,cursorpt,&cnp);
	cnp->Const_Type=CONST_MEET_CL;
	cnp->POutput=cursorpt;
	cnp->CInput1=ci1;
	cnp->LInput1=ln1;
	cnp->para1=CComplex(gx,0.0);
	cnp->para2=CComplex(gy,0.0);
	cnp->para3=CComplex(999.0,999.0);
	cnp->para4=CComplex(999.0,999.0);
	cnp->PInput1=NULL;
	GPoint* pt1;
	if(FindAnotherMeet(ci1,ln1,&pt1)){
		if(pt1!=cursorpt){
			cnp->PInput1=pt1;
		}
		else {
			cnp->PInput1=NULL;
		}
	}
	if(!CheckCoincidency(cursorpt)){
		AssignIncident(ln1,cursorpt);
		ResetLineIcPara(ln1);
		AssignIncident(ci1,cursorpt,atan2(- cursorpt->yz().real() + ci1->yy.real(),
					cursorpt->xz().real() - ci1->xx.real()));
		CheckIncidency(cursorpt);
	}
	return false;
}

bool MyCanvas::ConstMeet(GCircle* ci1, GCircle* ci2,double gx,double gy)
{
	//CONST_FREE??ONST_MEET_CC???X?B
	GConst * gcn;
	FindPtFromCn(cns,cursorpt,&gcn);
	gcn->Const_Type=CONST_MEET_CC;
	gcn->POutput=cursorpt;
	gcn->CInput1=ci1;
	gcn->CInput2=ci2;
	gcn->para1=CComplex(gx,0.0);
	gcn->para2=CComplex(gy,0.0);
	gcn->para3=CComplex(999.0,999.0);
	gcn->para4=CComplex(999.0,999.0);
	gcn->PInput1=NULL;
	GPoint* pt1;
	if(FindAnotherMeet(ci1,ci2,&pt1)){
		if(pt1!=cursorpt){
			gcn->PInput1=pt1;
		}
		else {
			gcn->PInput1=NULL;
		}
	}
	if(!CheckCoincidency(cursorpt)){
		AssignIncident(ci1,cursorpt,atan2(- cursorpt->yz().real() + ci1->yy.real(),
					cursorpt->xz().real() - ci1->xx.real()));
		AssignIncident(ci2,cursorpt,atan2(- cursorpt->yz().real() + ci2->yy.real(),
					cursorpt->xz().real() - ci2->xx.real()));
		CheckIncidency(cursorpt);
	}
	return false;
}

bool MyCanvas::DeletePtFromCn(GConst** cn,GPoint* pt)
{
	if(*cn==NULL){
		return false;
	}
	if((*cn)->POutput==pt){
		GConst* cn1= *cn;
		if((*cn)->next==NULL){
			lastcn = cn;
		}
		*cn = (*cn)->next;
		cn1->used=false;
		return true;
	}
	if((*cn)->next!=NULL){
		DeletePtFromCn(&((*cn)->next),pt);
	}
	return false;
}

bool MyCanvas::DeleteLnFromCn(GConst** cn,GLine* ln)
{
	if(*cn==NULL){
		return false;
	}
	if((*cn)->LOutput==ln){
		GConst* cn1= *cn;
		if((*cn)->next==NULL){
			lastcn = cn;
		}
		*cn = (*cn)->next;
		cn1->used=false;
		return true;
	}
	if((*cn)->next!=NULL){
		DeleteLnFromCn(&((*cn)->next),ln);
	}
	return false;
}

bool MyCanvas::DeleteCiFromCn(GConst** cn,GCircle* ci)
{
	if(*cn==NULL){
		return false;
	}
	if((*cn)->COutput==ci){
		GConst* cn1= *cn;
		if((*cn)->next==NULL){
			lastcn = cn;
		}
		*cn = (*cn)->next;
		cn1->used=false;
		return true;
	}
	if((*cn)->next!=NULL){
		DeleteCiFromCn(&((*cn)->next),ci);
	}
	return false;
}

bool MyCanvas::DeleteAllConstFromSheet(GConst *cn)	// Delete all const in a sheet
{
	if(cn==NULL){
		return false;
	}
	DeleteAllConstFromSheet(cn->next);
	switch(cn->Const_Type){
	case OUTPUT_POINT:
		DeletePoint(cn->POutput);
		break;
	case OUTPUT_LINE:
		DeleteLine(cn->LOutput);
		break;
	case OUTPUT_CIRCLE:
		DeleteCircle(cn->COutput);
		break;
	case OUTPUT_TEXT:
		DeleteText(cn->TextOutput);
		break;
	case OUTPUT_CURVE:
		DeleteCurve(cn->CurveOutput);
		break;
	}
	cn->next=NULL;
	cn->used=false;
	return false;
}

extern double moved_point_a11,moved_point_a12,moved_point_a21,moved_point_a22;

bool MyCanvas::EvaluateConst(GConst* cn)
{
	if(cn==NULL){
		return false;
	}
	else {
		cn->Evaluate();
		if(cn->POutput==moved_point && moved_point!=NULL){
			moved_point_a11=moved_point->a11;
			moved_point_a12=moved_point->a12;
			moved_point_a21=moved_point->a21;
			moved_point_a22=moved_point->a22;
		}
		return EvaluateConst(cn->next);
	}
}

void MyCanvas::ClearConst()
{
	int i;
	for(i=0;i<NUM_OF_POINTS;i++){
		pointbuffer[i].used=false;
		linebuffer[i].used=false;
		circlebuffer[i].used=false;
		textbuffer[i].used=false;
		if(i<50){
			curvebuffer[i].used=false;
		}
		constbuffer[i].used=false;
		sheetbuffer[i].used=false;
		constbuffer[i].POutput=NULL;
		constbuffer[i].LOutput=NULL;
		constbuffer[i].COutput=NULL;
		constbuffer[i].next=NULL;
		incidentbuffer[i]._used=false;
	}
	ClearGcals();
	sheetbuffer[0].MyPoint=pts=NULL;
	cursorpt=NULL;
	cursorpt2=NULL;
	sheetbuffer[0].MyLastPoint=lastpt=&pts;

	sheetbuffer[0].MyLine=lns=NULL;
	cursorln=NULL;
	sheetbuffer[0].MyLastLine=lastln=&lns;


	cns=NULL;
	cursorcn=NULL;
	lastcn=&cns;
	sheetbuffer[0].MyConst=NULL;
	sheetbuffer[0].MyLastConst=&(sheetbuffer[0].MyConst);

	sheetbuffer[0].MyCircle=cis=NULL;
	cursorci=NULL;
	sheetbuffer[0].MyLastCircle=lastci=&cis;

	sheetbuffer[0].MyText=texts=NULL;
	cursortext=NULL;
	sheetbuffer[0].MyLastText=lasttext=&texts;

	sheetbuffer[0].MyParaCurve=curves=NULL;
	cursorcurve=NULL;
	sheetbuffer[0].MyLastParaCurve=lastcurve=&curves;

	NOMySheet=1;
	currentSheet=0;
	
	ResetAllName();
	LineClipping=false;

}

//
//   class GSheet
//


GSheet::GSheet(void)
{
	MyConst=NULL;
	MyPoint=NULL;
	MyLine=NULL;
	MyCircle=NULL;
	MyText=NULL;
	MyParaCurve=NULL;
	SheetNumber=0;
}

GSheet::~GSheet(void)
{
}

GSheet& GSheet::operator=(const GSheet& z)
{
	used=z.used;
	MyConst=z.MyConst;
	MyLastConst=z.MyLastConst;
	MyPoint=z.MyPoint;
	MyLastPoint=z.MyLastPoint;
	MyLine=z.MyLine;
	MyLastLine=z.MyLastLine;
	MyCircle=z.MyCircle;
	MyLastCircle=z.MyLastCircle;
	MyText=z.MyText;
	MyLastText=z.MyLastText;
	MyParaCurve=z.MyParaCurve;
	MyLastParaCurve=z.MyLastParaCurve;
	SheetNumber=z.SheetNumber;
	return *this;
}

void MyCanvas::InsertNewSheet(void)
{
	int i;
	for(i=NOMySheet-1;i>=currentSheet+1;i--){
		sheetbuffer[i+1]=sheetbuffer[i];
		sheetbuffer[i+1].SheetNumber=sheetbuffer[i].SheetNumber + 1;  // ????s?v
	}
	NOMySheet++;
	currentSheet++;
	cns = sheetbuffer[currentSheet].MyConst=NULL;
	lastcn = sheetbuffer[currentSheet].MyLastConst = &cns;
	pts = sheetbuffer[currentSheet].MyPoint=NULL;
	lastpt = sheetbuffer[currentSheet].MyLastPoint = &pts;
	lns = sheetbuffer[currentSheet].MyLine=NULL;
	lastln = sheetbuffer[currentSheet].MyLastLine = &lns;
	cis = sheetbuffer[currentSheet].MyCircle=NULL;
	lastci = sheetbuffer[currentSheet].MyLastCircle = &cis;
	texts = sheetbuffer[currentSheet].MyText=NULL;
	lasttext = sheetbuffer[currentSheet].MyLastText = &texts;
	curves = sheetbuffer[currentSheet].MyParaCurve=NULL;
	lastcurve = sheetbuffer[currentSheet].MyLastParaCurve = &curves;

	ResetAllName();
	Refresh(false);
}

void MyCanvas::InsertCopyFigure(bool texton)
{
	int i;
	GConst* oldcns;
	GPoint* oldpts;
	GLine*  oldlns;
	GCircle*oldcis;
	GText*  oldtexts;
	GParaCurve* oldcurves;
	for(i=NOMySheet-1;i>=currentSheet+1;i--){
		sheetbuffer[i+1]=sheetbuffer[i];
		sheetbuffer[i+1].SheetNumber=sheetbuffer[i].SheetNumber + 1;
	}
	oldcns=sheetbuffer[currentSheet].MyConst;
	oldpts=sheetbuffer[currentSheet].MyPoint;
	oldlns=sheetbuffer[currentSheet].MyLine;
	oldcis=sheetbuffer[currentSheet].MyCircle;
	oldtexts=sheetbuffer[currentSheet].MyText;
	oldcurves=curves = sheetbuffer[currentSheet].MyParaCurve;
	NOMySheet++;
	currentSheet++;
	cns = sheetbuffer[currentSheet].MyConst=NULL;
	lastcn = sheetbuffer[currentSheet].MyLastConst = &cns;
	pts = sheetbuffer[currentSheet].MyPoint=NULL;
	lastpt = sheetbuffer[currentSheet].MyLastPoint = &pts;
	lns = sheetbuffer[currentSheet].MyLine=NULL;
	lastln = sheetbuffer[currentSheet].MyLastLine = &lns;
	cis = sheetbuffer[currentSheet].MyCircle=NULL;
	lastci = sheetbuffer[currentSheet].MyLastCircle = &cis;
	texts = sheetbuffer[currentSheet].MyText=NULL;
	lasttext = sheetbuffer[currentSheet].MyLastText = &texts;
	curves = sheetbuffer[currentSheet].MyParaCurve=NULL;
	lastcurve = sheetbuffer[currentSheet].MyLastParaCurve = &curves;
	ResetAllName();

	const_src=wxT("");
	if(!SetConstSrc(oldcns)){
		NOConstStock=0;
		const_src2cns(false,texton);
	}
	Refresh(false);
}
void MyCanvas::DeleteSheet()
{
	ClearSheet();
	for(int i=currentSheet;i<NOMySheet-1;i++){
		sheetbuffer[i]=sheetbuffer[i+1];
		sheetbuffer[i].SheetNumber=sheetbuffer[i+1].SheetNumber - 1;  
	}
	NOMySheet--;
	cns= sheetbuffer[currentSheet].MyConst;	
	lastcn= sheetbuffer[currentSheet].MyLastConst;
	pts= sheetbuffer[currentSheet].MyPoint;	
	lastpt= sheetbuffer[currentSheet].MyLastPoint;
	lns = sheetbuffer[currentSheet].MyLine;
	lastln = sheetbuffer[currentSheet].MyLastLine;
	cis = sheetbuffer[currentSheet].MyCircle;
	lastci = sheetbuffer[currentSheet].MyLastCircle ;
	texts = sheetbuffer[currentSheet].MyText;
	lasttext = sheetbuffer[currentSheet].MyLastText;
	curves = sheetbuffer[currentSheet].MyParaCurve;
	lastcurve = sheetbuffer[currentSheet].MyLastParaCurve;
	ResetAllName();
	EvaluateConst(cns);
	Refresh(false);
}
void MyCanvas::ClearSheet()
{
	for(GPoint* ptp=pts;ptp!=NULL;ptp=ptp->next){
		ptp->used=false;
	}
	for(GLine* lnp=lns;lnp!=NULL;lnp=lnp->next){
		DeleteIcs(lnp);
		lnp->used=false;
	}
	for(GCircle* cip=cis;cip!=NULL;cip=cip->next){
		DeleteIcs(cip);
		cip->used=false;
	}
	for(GParaCurve* curvep=curves;curvep!=NULL;curvep=curvep->next){
		curvep->used=false;
	}
	for(GText* textp=texts;textp!=NULL;textp=textp->next){
		textp->used=false;
	}
}
bool MyCanvas::NextSheet(void)
{
	if(currentSheet<0 || currentSheet>=NOMySheet-1){
		return false;
	}
	currentSheet++;
	cns= sheetbuffer[currentSheet].MyConst;	
	lastcn= sheetbuffer[currentSheet].MyLastConst;
	pts= sheetbuffer[currentSheet].MyPoint;	
	lastpt= sheetbuffer[currentSheet].MyLastPoint;
	lns = sheetbuffer[currentSheet].MyLine;
	lastln = sheetbuffer[currentSheet].MyLastLine;
	cis = sheetbuffer[currentSheet].MyCircle;
	lastci = sheetbuffer[currentSheet].MyLastCircle ;
	texts = sheetbuffer[currentSheet].MyText;
	lasttext = sheetbuffer[currentSheet].MyLastText;
	curves = sheetbuffer[currentSheet].MyParaCurve;
	lastcurve = sheetbuffer[currentSheet].MyLastParaCurve;
	ResetAllName();
	return true;

}

void MyCanvas::PrevSheet(void)
{
	if(currentSheet<=0 || currentSheet>NOMySheet-1){
		return ;
	}
	currentSheet--;
	cns= sheetbuffer[currentSheet].MyConst;	
	lastcn= sheetbuffer[currentSheet].MyLastConst;
	pts= sheetbuffer[currentSheet].MyPoint;	
	lastpt= sheetbuffer[currentSheet].MyLastPoint;
	lns = sheetbuffer[currentSheet].MyLine;
	lastln = sheetbuffer[currentSheet].MyLastLine;
	cis = sheetbuffer[currentSheet].MyCircle;
	lastci = sheetbuffer[currentSheet].MyLastCircle ;
	texts = sheetbuffer[currentSheet].MyText;
	lasttext = sheetbuffer[currentSheet].MyLastText;
	curves = sheetbuffer[currentSheet].MyParaCurve;
	lastcurve = sheetbuffer[currentSheet].MyLastParaCurve;
	ResetAllName();
}

wxString ConvertText4Script(wxString& st)
{

	int j;
	char buff[1024];
	int offset=0;
	char*p=(char*)st.c_str();
	int len=strlen(p);
	for(j=0;j<len;j++){
		if(p[j]=='\n') {
			buff[offset]='\\';offset++;
			buff[offset]='n';offset++;
		}
		else if(p[j]=='\r') {
			;
		}
		else {
			buff[offset]=p[j];offset++;
		}
	}
	buff[offset]=0x00;
	return wxString((wxChar*)buff);
}

wxString MyCanvas::ConvertTextFromScript(wxString& st)
{

	int j;
	char buff[1024];
	int offset=0;
	char*p=(char*)st.c_str();
	int len=strlen(p);
	for(j=0;j<len;j++){
		if(p[j]=='\\' && p[j+1]=='n'){
			buff[offset]='\n';offset++;
			j++;
		}
		else {
			buff[offset]=p[j];offset++;
		}
	}
	buff[offset]=0x00;
	return wxString((wxChar*)buff);
}

bool MyCanvas::duplicate_line(GLine* lnp, GLine* ln){
	if(lnp==NULL || ln==NULL){
		return false;
	}
	if(lnp==ln){
		return false;
	}
	else if(lnp->xx * ln->yy - ln->xx * lnp->yy == CComplex(0.0,0.0) && lnp->zz * ln->yy - ln->zz * lnp->yy == CComplex(0.0,0.0)){
		return true;
	}
	else {
		return duplicate_line(lnp->next,ln);
	}
}

extern wxBitmap		*gs_bmpScreen;
extern wxBitmap		*gs_bmpBackGround;

void MyCanvas::OnBenchMark(void)
{
	unsigned long t1,t2,t3,t4;
	wxStartTimer();
	for(int i=0;i<1000;i++){
		EvaluateConst(cns);
	}
	t1=wxGetElapsedTime(false);
	wxStartTimer();
	for(int i=0;i<1000;i++){
		g_memBGdc.SelectObject(*gs_bmpBackGround);
		DrawBackGround(&g_memBGdc);
	}
	t2=wxGetElapsedTime(false);
	wxStartTimer();
	for(int i=0;i<1000;i++){
		g_memdc.SelectObject(*gs_bmpScreen);
		OnDraw(&g_memdc);
	}
	t3=wxGetElapsedTime(false);
	wxStartTimer();
	for(int i=0;i<1000;i++){
		wxPaintDC dc(this);
		PrepareDC(dc);
		m_owner->PrepareDC(dc);
		dc.Blit(0,0,windowwidth,windowheight,&g_memdc,0,0);
	}
	t4=wxGetElapsedTime(false);
	wxString message;
	message.Printf(wxT("evaluate const. = %ld(us)\ndraw background = %ld(us)\ndraw elements= %ld(us)\nblit all = %ld(us)\n"),t1,t2,t3,t4);
	wxMessageBox(message,wxT("Benchmark Test"),wxOK | wxICON_INFORMATION, this);
}
