// printing.cpp

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/image.h"
#include "wx/accel.h"

#include "Printing.h"
#include "KidsCindyFrame.h"
#include "KidsCindyCanvas.h"

#if wxUSE_LIBGNOMEPRINT
#include "wx/html/forcelnk.h"
FORCE_LINK(gnome_print)
#endif

// Global print data, to remember settings during the session
wxPrintData *g_printData = (wxPrintData*) NULL ;

// Global page setup data
wxPageSetupData* g_pageSetupData = (wxPageSetupData*) NULL;

// Writes a header on a page. Margin units are in millimetres.
bool WritePageHeader(wxPrintout *printout, wxDC *dc, wxChar *text, float mmToLogical);



void MyFrame::InitPrintData()
{
	g_printData = new wxPrintData;
	g_pageSetupData = new wxPageSetupDialogData;
}

void MyFrame::DeletePrintData()
{
	delete g_printData;
	delete g_pageSetupData;
}

void MyFrame::OnPrint(wxCommandEvent& WXUNUSED(event))
{
	InitPrintData();
	wxPrintDialogData printDialogData(* g_printData);

	wxPrinter printer(& printDialogData);
	MyPrintout printout(_T("My printout"), m_canvas);
	if (!printer.Print(this, &printout, true /*prompt*/))
	{
		if (wxPrinter::GetLastError() == wxPRINTER_ERROR)
			wxMessageBox(_T("There was a problem printing.\nPerhaps your current printer is not set correctly?"), _T("Printing"), wxOK);
		else
			wxMessageBox(_T("You canceled printing"), _T("Printing"), wxOK);
	}
	else
	{
		(*g_printData) = printer.GetPrintDialogData().GetPrintData();
	}
	DeletePrintData();
}

void MyFrame::OnPrintPreview(wxCommandEvent& WXUNUSED(event))
{
	InitPrintData();
	// Pass two printout objects: for preview, and possible printing.
	wxPrintDialogData printDialogData(* g_printData);
	wxPrintPreview *preview = new wxPrintPreview(new MyPrintout(_T("My printout"), m_canvas), new MyPrintout(_T("My printout"), m_canvas), & printDialogData);
	if (!preview->Ok())
	{
		delete preview;
		wxMessageBox(_T("There was a problem previewing.\nPerhaps your current printer is not set correctly?"), _T("Previewing"), wxOK);
		DeletePrintData();
		return;
	}

	wxPreviewFrame *frame = new wxPreviewFrame(preview, this, _T("Demo Print Preview"), wxPoint(100, 100), wxSize(600, 650));
	frame->Centre(wxBOTH);
	frame->Initialize();
	frame->Show();
	DeletePrintData();
}

void MyFrame::OnPageSetup(wxCommandEvent& WXUNUSED(event))
{
	InitPrintData();
	(*g_pageSetupData) = *g_printData;

	wxPageSetupDialog pageSetupDialog(this, g_pageSetupData);
	pageSetupDialog.ShowModal();

	(*g_printData) = pageSetupDialog.GetPageSetupData().GetPrintData();
	(*g_pageSetupData) = pageSetupDialog.GetPageSetupData();
	DeletePrintData();
}

MyPrintout::MyPrintout(wxChar *title,MyCanvas * canvas):wxPrintout(title)
{ 
	granpa = canvas; 
	geomperdot = canvas->geomperdot;
	boxright=canvas->boxright;
	boxleft=canvas->boxleft;
	boxtop=canvas->boxtop;
	boxbottom=canvas->boxbottom;
}


bool MyPrintout::OnPrintPage(int page)
{
	wxDC *dc = GetDC();
	if (dc)
	{
		if (page == 1)
			DrawPage(dc);

		dc->SetDeviceOrigin(0, 0);
		dc->SetUserScale(1.0, 1.0);

		wxChar buf[200];
		wxSprintf(buf, wxT("PAGE %d"), page);
		dc->DrawText(buf, 10, 10);

		return true;
	}
	else
		return false;
}

bool MyPrintout::OnBeginDocument(int startPage, int endPage)
{
	if (!wxPrintout::OnBeginDocument(startPage, endPage))
		return false;

	return true;
}

void MyPrintout::GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo)
{
	*minPage = 1;
	*maxPage = 1;
	*selPageFrom = 1;
	*selPageTo = 1;
}

bool MyPrintout::HasPage(int pageNum)
{
	return (pageNum == 1);
}

void MyPrintout::DrawPage(wxDC *dc)
{
	// You might use THIS code if you were scaling
	// graphics of known size to fit on the page.
	float maxX = (boxright - boxleft) / geomperdot;
	float maxY = (boxtop - boxbottom) / geomperdot;

	// Let's have at least 50 device units margin
	float marginX = 50;
	float marginY = 50;

	// Add the margin to the graphic size
	maxX += (2*marginX);
	maxY += (2*marginY);

	// Get the size of the DC in pixels
	int w, h;
	dc->GetSize(&w, &h);

	// Calculate a suitable scaling factor
	float scaleX=(float)(w/maxX);
	float scaleY=(float)(h/maxY);

	// Use x or y scaling factor, whichever fits on the DC
	float actualScale = wxMin(scaleX,scaleY);

	// Calculate the position on the DC for centring the graphic
	float posX = 50.0*actualScale;
	float posY = 50.0*actualScale;

	// Set the scale and origin
	dc->SetUserScale(actualScale, actualScale);
	dc->SetDeviceOrigin( (long)posX, (long)posY );
//	dc->SetDeviceOrigin(0,0);
	int thm = granpa->ThemeMode;
	switch(thm){
	case THM_STANDARD:
	case THM_CAT:
	case THM_STAR:
	case THM_SKELETON:
		granpa->SetTheme(THM_CINDY);
	}
	granpa->OnDraw((wxMemoryDC* )dc);
	switch(thm){
	case THM_STANDARD:
	case THM_CAT:
	case THM_STAR:
	case THM_SKELETON:
		granpa->SetTheme(thm);
	}
}

// Writes a header on a page. Margin units are in millimetres.
bool WritePageHeader(wxPrintout *printout, wxDC *dc, wxChar *text, float mmToLogical)
{
/*
static wxFont *headerFont = (wxFont *) NULL;
if (!headerFont)
{
headerFont = wxTheFontList->FindOrCreateFont(16, wxSWISS, wxNORMAL, wxBOLD);
}
dc->SetFont(headerFont);
	*/

	int pageWidthMM, pageHeightMM;

	printout->GetPageSizeMM(&pageWidthMM, &pageHeightMM);
//	wxUnusedVar(pageHeightMM);

	int leftMargin = 10;
	int topMargin = 10;
	int rightMargin = 10;

	float leftMarginLogical = (float)(mmToLogical*leftMargin);
	float topMarginLogical = (float)(mmToLogical*topMargin);
	float rightMarginLogical = (float)(mmToLogical*(pageWidthMM - rightMargin));

	long xExtent, yExtent;
	dc->GetTextExtent(text, &xExtent, &yExtent);
	float xPos = (float)(((((pageWidthMM - leftMargin - rightMargin)/2.0)+leftMargin)*mmToLogical) - (xExtent/2.0));
	dc->DrawText(text, (long)xPos, (long)topMarginLogical);

	dc->SetPen(* wxBLACK_PEN);
	dc->DrawLine( (long)leftMarginLogical, (long)(topMarginLogical+yExtent),
		(long)rightMarginLogical, (long)topMarginLogical+yExtent );

	return true;
}
